var socket = io('https://crm.solus.se');
var socketInit = false;
var reconnecting = false;
var user = null;
var pass = null;
var loggedin = false;
var nopop = false;

function activeTab(cb) {
    chrome.tabs.query({active: true, currentWindow: true}, function(tabs) {
        if(tabs.length > 0) {
            cb(tabs[0].id);
        } else
            cb(null);
    });
}

var caller = null;

function connect() {
    if (!socketInit) {
        socketInit = true;

        //attach event handlers
        socket.on('connect', function () {
            console.log('connect fired!');
            if (user && reconnecting) {
                console.log('logging in');
                socket.emit('hi', {
                    user: user,
                    pass: pass,
                    format: 'html-small',
                    lines: 4
                });
                reconnecting = false;
            }
        });

        socket.on('disconnect', function () {
            console.log('disconnect fired!');
            
            loggedin = false;
            chrome.browserAction.setIcon({
                path: 'img/phone-call-icon-16-terminated.png'
            });
        });

        socket.on('reconnect', function () {
            console.log('reconnect fired!');
            reconnecting = true;
        });
        
        socket.on('notification-caller', function (msg) {
            console.log('got caller notification');
            caller = msg;
        });

        socket.on('notification', function (msg) {
            console.log('got call notification');
            
            chrome.storage.local.set({'lastmsg': msg}, function() {

                activeTab(function(tabid) {
                    if(tabid) {
                        chrome.tabs.sendMessage(tabid, "display");
                    } else {
                        console.log('NO TAB - notifying');
                        /*
                        chrome.notifications.create('call', {
                            "type": "basic",
                            "iconUrl": chrome.extension.getURL("img/phone-call-icon-16.png"),
                            "title": caller,
                            "message": "Ringer"
                        });
                        */
                    }
                });

            });

        });

        socket.on('external', function (url) {
            console.warn('got call that we should handle with an external URL');

            chrome.windows.create({'url': url, 'type': 'popup'}, function(window) {
                console.log('done!')
            });
        });


        socket.on('call-early', function (msg) {
            console.warn('got call');
            
            chrome.storage.local.set({'lastmsg': msg}, function() {
                activeTab(function(tabid) {
                    if(tabid) {
                        chrome.tabs.sendMessage(tabid, "display");
                    }
                });
            });

        });
                    
        socket.on('call-confirmed', function (msg) {
            console.warn('got confirmed');

            chrome.storage.local.set({'lastmsg': msg}, function() {
                activeTab(function(tabid) {
                    if(tabid) {
                        chrome.tabs.sendMessage(tabid, "display");
                    }
                });
            });

        });
        
        socket.on('call-terminated', function (msg) {
            console.warn('got terminated');
            
            chrome.storage.local.set({'lastmsg': msg}, function() {
                activeTab(function(tabid) {
                    if(tabid) {
                        chrome.tabs.sendMessage(tabid, "display-terminated");
                    }
                });
            });
        });

        socket.on('login-success', function (msg) {
            //console.log(msg);
            
            loggedin = true;
            chrome.storage.local.set({'error': null, 'loggedin': true}, function() {
                chrome.browserAction.setIcon({
                    path: '/img/phone-call-icon-16-answered.png'
                });
            });
        });

        socket.on('login-failed', function (msg) {
            console.log(msg);
            
            loggedin = false;
            chrome.storage.local.set({'error': Date.now(), 'loggedin': false}, function() {
                chrome.browserAction.setIcon({
                    path: '/img/phone-call-icon-16-terminated.png'
                });
            });
            
        });

    } else {
        socket.io.reconnect();
    }
}

function disconnect() {
    socket.io.disconnect();
}

chrome.runtime.onMessage.addListener(function(message, sender) {
    if(message == 'pop') {
        chrome.storage.local.set({nopop: false});
    } else if(message == 'nopop') {
        chrome.storage.local.set({nopop: true});
    } else {
        chrome.tabs.sendMessage(sender.tab.id, message);
    }
});

chrome.browserAction.onClicked.addListener(function(tab) {
    if(loggedin) {
        chrome.tabs.sendMessage(tab.id, 'toggle_popup');
    } else {
        chrome.runtime.openOptionsPage();
    }
});

chrome.storage.onChanged.addListener(function(changes, area) {
    if(changes.nopop) {
        chrome.storage.local.get(['nopop'], function(items) {
            nopop = items.nopop;
        });
    }
    if (changes.user || changes.pass) {
        console.log('logging in!');
        
        chrome.storage.local.get(['user', 'pass'], function(items) {
            user = items.user;
            pass = items.pass;

            if(user) {
                console.log('try again');

                socket.emit('hi', {
                    user: user,
                    pass: pass,
                    format: 'html-small',
                    lines: 4
                });
            } else {
                console.log('log out');

                socket.emit('bye');

                loggedin = false;
                chrome.storage.local.set({'loggedin': false}, function() {
                    chrome.browserAction.setIcon({
                        path: '/img/phone-call-icon-16-terminated.png'
                    });
                });
            }
        });
    }
});

chrome.storage.local.get(['user', 'pass', 'nopop'], function(items) {
    console.log('connecting');
    user = items.user;
    pass = items.pass;
    nopop = items.nopop;
    if(user) {
        reconnecting = true;
    }
    
    connect();
});
